"Grant Jenks Tools main entry point."
# pylint: disable=invalid-name

import argparse
import logging

from . import packages, readinglist, utils

parser = argparse.ArgumentParser(
    'gj',
    formatter_class=argparse.ArgumentDefaultsHelpFormatter,
)
parser.add_argument('-d', '--debug', action='store_true')
parser.add_argument('-v', '--verbose', action='store_true')

subparsers = parser.add_subparsers(dest='subcommand', help='<subcommand>')

parser_release = subparsers.add_parser('release', help='release package')
parser_release.add_argument('--no-pylint', action='store_true')
parser_release.add_argument('--no-tox', action='store_true')
parser_release.add_argument('--no-docs', action='store_true')

parser_upload_docs = subparsers.add_parser('upload-docs', help='upload docs')
parser_upload_docs.add_argument('name')

parser_reading_list = subparsers.add_parser(
    'reading-list',
    help='reading list',
)
parser_reading_list.add_argument('-c', '--clear', action='store_true')

parser_watch = subparsers.add_parser(
    'watch',
    help='watch files for changes and run command',
)
parser_watch.add_argument('command')
parser_watch.add_argument('filename', nargs='+')

args = parser.parse_args()

if args.debug:
    log_level = logging.DEBUG
elif args.verbose:
    log_level = logging.INFO
else:
    log_level = logging.WARNING

logging.basicConfig(level=log_level)

if args.subcommand == 'release':
    packages.release(
        pylint=not args.no_pylint,
        tox=not args.no_tox,
        docs=not args.no_docs,
    )
elif args.subcommand == 'upload-docs':
    packages.upload_docs(
        name=args.name,
    )
elif args.subcommand == 'reading-list':
    readinglist.read(
        clear=args.clear,
    )
elif args.subcommand == 'watch':
    utils.watch(
        command=args.command.split(),
        filenames=args.filename,
    )



"""Read from iCloud Reading List

"""

from __future__ import print_function


def lookup(data, *args):
    "Return result of sequence of lookups on data."
    for arg in args:
        if isinstance(arg, int):
            data = data[arg]
        elif isinstance(arg, str):
            data = data[arg]
        elif isinstance(arg, tuple):
            key, value = arg
            for item in data:
                if item[key] == value:
                    data = item
                    break
            else:
                raise LookupError
    return data


def read(clear=False):
    "Read item from reading list."
    import biplist  # pylint: disable=import-error
    import diskcache  # pylint: disable=import-error
    import os.path
    import webbrowser

    home_dir = os.path.expanduser('~')
    gj_dir = os.path.join(home_dir, '.gj')
    cache_dir = os.path.join(gj_dir, 'diskcache')
    cache = diskcache.FanoutCache(cache_dir)
    reading_list_index = cache.index('reading-list')

    if clear:
        reading_list_index.clear()
        return

    data = biplist.readPlist('/Users/grantj/Library/Safari/Bookmarks.plist')

    reading_list_all = lookup(
        data,
        'Children',
        ('Title', 'com.apple.ReadingList'),
        'Children'
    )

    reading_list = [
        item for item in reading_list_all
        if 'DateLastViewed' not in item['ReadingList']
    ]

    reading_list.sort(key=lambda item: item['ReadingList']['DateAdded'])

    for item in reading_list:
        url = item['URLString']

        if url not in reading_list_index:
            reading_list_index[url] = 1
            webbrowser.open(url)
            break
    else:
        print(u'Nothing to read \N{white frowning face}')



"Packages tools."

from __future__ import print_function

import glob
import os
import os.path as op
import re
import shutil
import subprocess as sp
import sys


def run(command):
    "Run `command` and display output."
    print('gj$', command)
    sp.check_call(command.split())


def chdir(path):
    "Change working directory to `path`."
    print('gj$ cd', path)
    os.chdir(path)


def sftp_mkdir(sftp, path):
    "Make directory at `path` using `sftp` connection."
    print('gj$ # sftp.mkdir', path)
    try:
        sftp.mkdir(path)
    except IOError:
        pass


def sftp_upload(sftp, local_path, remote_path):
    "Upload binary file `reader` to `path` using `sftp` connection."
    print('gj$ # sftp.put', local_path, remote_path)
    sftp.put(local_path, remote_path)


def lookup_name(cwd):
    "Lookup package name in directory `cwd`."

    _, dirname = op.split(cwd)

    prefixes = ['python-', 'python_', 'django-']

    for prefix in prefixes:
        if dirname.startswith(prefix):
            dirname = dirname[len(prefix):]
            return dirname.replace('_', '')

    if '-python-' in dirname:
        return dirname.replace('-python-', '')


def lookup_version(name):
    "Lookup version for `name` package."

    try:
        with open(op.join(name, '__init__.py')) as reader:
            lines = reader.readlines()
    except IOError:
        with open('%s.py' % name) as reader:
            lines = reader.readlines()

    for line in lines:
        match = re.match(r'^__version__ = \'(.*)\'$', line)
        if match:
            return match.group(1)

    print('Error: Unknown version.')
    sys.exit(1)


def upload_docs_v0(name):
    "Upload docs for package with `name`."
    print('gj$ # Uploading Docs')

    import paramiko
    ssh = paramiko.SSHClient()
    ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
    ssh.connect('104.198.0.77')
    sftp = ssh.open_sftp()

    base = '/srv/www/www.grantjenks.com/docs/%s' % name

    sftp_mkdir(sftp, base)

    chdir(op.join('_build', 'html'))

    for path, dirs, files in os.walk('.'):
        for directory in dirs:
            sftp_mkdir(sftp, '/'.join([base, path, directory]))

        for filename in files:
            local_path = op.join(path, filename)
            remote_path = '/'.join([base, path, filename])
            sftp_upload(sftp, local_path, remote_path)


def upload_docs(name):
    "Upload docs for package with `name`."
    base = '/srv/www/www.grantjenks.com/docs/%s' % name
    rsync = 'rsync -azP --stats --delete _build/html/ magnesium:%s' % base
    run(rsync)


def release(pylint=True, tox=True, docs=True):
    "Release package"
    cwd = os.getcwd()
    name = lookup_name(cwd)
    version = lookup_version(name)
    version = 'v%s' % version

    run('git checkout master')

    if version in sp.check_output(['git', 'tag']).decode('utf-8'):
        print('Error: Version already tagged.')
        sys.exit(1)

    if sp.check_output(['git', 'status', '--porcelain']):
        print('Error: Commit files in working directory before release.')
        run('git status')
        sys.exit(1)

    run('git pull')

    if pylint:
        run('pylint %s' % name)

    if tox:
        run('tox --skip-missing-interpreters')

    run('rstcheck --report warning README.rst')
    run('doc8 docs')

    run('git tag -a %s -m %s' % (version, version))
    run('git push')
    run('git push --tags')

    shutil.rmtree('dist', ignore_errors=True)

    setup_py = open('setup.py').read()

    if 'Extension(' in setup_py:
        dist = 'sdist'  # Use source distribution for binary extensions.
    else:
        dist = 'sdist bdist_wheel --universal'

    run('python setup.py %s' % dist)
    run('twine upload ' + ' '.join(glob.glob('dist/*')))

    if not docs:
        return

    shutil.rmtree(op.join('docs', '_build'), ignore_errors=True)

    print('gj$ # Building Docs')

    chdir(op.join(cwd, 'docs'))

    run('make clean')
    run('make html')

    upload_docs(name)

    chdir(op.join(cwd, 'docs'))
    run('make clean')

    chdir(cwd)
    run('rm -r build')



"Grant Jenks tools."

from .packages import release, upload_docs
from .utils import watch

__title__ = 'gj'
__version__ = '0.3.1'
__build__ = 0x000301
__author__ = 'Grant Jenks'
__license__ = 'Apache 2.0'
__copyright__ = 'Copyright 2016 Grant Jenks'



"""Miscellaneous Utilities

"""

from __future__ import print_function

import logging
import os
import subprocess
import time

log = logging.getLogger(__name__)  # pylint: disable=invalid-name


def watch(command, filenames):
    """Run `command` when any of `filenames` are modified.

    """
    filenames = sorted(set(filenames))
    mtimes = {}

    for filename in filenames:
        try:
            mtime = os.path.getmtime(filename)
            mtimes[filename] = mtime
            log.debug('Watching %s %r', mtime, filename)
        except OSError:
            log.error('mtime error for %r', filename)
            return

    try:
        while True:
            changed = False

            for filename in filenames:
                mtime = os.path.getmtime(filename)

                if mtimes[filename] != mtime:
                    log.info('Detected mtime change for %r', filename)
                    changed = True
                    mtimes[filename] = mtime

            if changed:
                log.info('Running %r', command)
                subprocess.call(command)

            time.sleep(1)
    except KeyboardInterrupt:
        print()
        exit(0)



from setuptools import setup, find_packages
from setuptools.command.test import test as TestCommand
import sys

import gj


class Tox(TestCommand):
    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True
    def run_tests(self):
        import tox
        errno = tox.cmdline(self.test_args)
        sys.exit(errno)


with open('README.rst') as reader:
    readme = reader.read()

with open('LICENSE') as reader:
    license = reader.read()


setup(
    name='gj',
    version=gj.__version__,
    description='Disk and file-based cache',
    long_description=readme,
    author='Grant Jenks',
    author_email='contact@grantjenks.com',
    url='http://www.grantjenks.com/docs/gj/',
    packages=find_packages(exclude=('tests', 'docs')),
    package_data={'': ['LICENSE', 'README.rst']},
    tests_require=['tox'],
    cmdclass={'test': Tox},
    license='Apache 2.0',
    install_requires=[],
    keywords='grant jenks tools',
    classifiers=(
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Natural Language :: English',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: Implementation :: CPython',
    ),
)
